Package: TestGenerator
Type: Package
Title: Integration Unit Tests for Pharmacoepidemiological Studies
Version: 0.8.0
Authors@R: c(
    person("Cesar", "Barboza", , "c.barboza@darwin-eu.org", role = c("aut", "cre"), comment = c(ORCID = "0009-0002-4453-3071")),
    person("Ioanna", "Nika", , "i.nika@darwin-eu.org", role = "aut"),
    person("Ger", "Inberg", , "g.inberg@erasmusmc.nl", role = c("aut"), comment = c(ORCID = "0000-0001-8993-8748")),
    person("Adam", "Black", , "black@ohdsi.org", role = "aut", comment = c(ORCID = "0000-0001-5576-8701"))
  )
Maintainer: Cesar Barboza <c.barboza@darwin-eu.org>
Description: An R interface to load testing data in the 'OMOP' Common Data Model ('CDM'). An input file, csv or xlsx, can be converted to a 'CDMConnector' object. This object can be used to execute and test studies that use the 'CDM' <https://www.ohdsi.org/data-standardization/>.
License: Apache License (>= 2)
Encoding: UTF-8
RoxygenNote: 7.3.3
Depends: R (>= 4.1.0)
Imports: jsonlite, readxl, readr, CDMConnector (>= 2.5.1), DBI, dplyr,
        checkmate, glue, duckdb, cli, rlang, withr, tibble, testthat,
        openxlsx, omopgenerics, RPostgres, fs, stringr, usethis
Suggests: knitr, rmarkdown, curl, httr, odbc, CohortConstructor
VignetteBuilder: knitr
Config/testthat/edition: 3
URL: https://github.com/darwin-eu/TestGenerator,
        https://darwin-eu.github.io/TestGenerator/
BugReports: https://github.com/darwin-eu/TestGenerator/issues
NeedsCompilation: no
Packaged: 2026-05-17 22:00:45 UTC; runner
Author: Cesar Barboza [aut, cre] (ORCID:
    <https://orcid.org/0009-0002-4453-3071>),
  Ioanna Nika [aut],
  Ger Inberg [aut] (ORCID: <https://orcid.org/0000-0001-8993-8748>),
  Adam Black [aut] (ORCID: <https://orcid.org/0000-0001-5576-8701>)
Repository: CRAN
Date/Publication: 2026-05-26 13:20:21 UTC
Built: R 4.6.0; ; 2026-05-26 16:08:50 UTC; unix
