## ----setup, include = FALSE---------------------------------------------------
knitr::opts_chunk$set(
  collapse = TRUE,
  comment = "#>",
  eval = FALSE
)

## ----all-actions--------------------------------------------------------------
# library(TestGenerator)
# 
# useGithubAction(overwrite = TRUE)

## ----one-action---------------------------------------------------------------
# useGithubAction(dbms_type = "postgresql", overwrite = TRUE)
# useGithubAction(dbms_type = "sqlserver", overwrite = TRUE)
# useGithubAction(dbms_type = "databricks", overwrite = TRUE)

