% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/useGithubAction.R
\name{useGithubAction}
\alias{useGithubAction}
\title{Create GitHub Action workflows for specific DBMS}
\usage{
useGithubAction(
  dbms_type = c("postgresql", "sqlserver", "databricks"),
  overwrite = FALSE
)
}
\arguments{
\item{dbms_type}{A character vector of supported DBMS types to process.
Supported values are \code{"postgresql"}, \code{"sqlserver"}, and
\code{"databricks"}.}

\item{overwrite}{A logical value indicating whether to overwrite existing
workflow files in the \code{.github/workflows} directory. Defaults to
\code{FALSE}.}
}
\value{
Invisibly returns \code{TRUE} when the selected workflow files are
copied successfully.
}
\description{
Copies GitHub Action workflow templates from the package's
\code{inst/workflows} directory to the \code{.github/workflows} directory of
the current project. The copied workflows can be used to run backend-specific
tests for PostgreSQL, SQL Server, and Databricks.
}
\examples{
\dontrun{
useGithubAction(dbms_type = "postgresql")
useGithubAction(
  dbms_type = c("postgresql", "sqlserver", "databricks"),
  overwrite = TRUE
)
}
}
