% Generated by roxygen2: do not edit by hand
% Please edit documentation in R/patientsRemote.R
\name{cleanupTestCdm}
\alias{cleanupTestCdm}
\title{Clean up a test CDM on a remote database}
\usage{
cleanupTestCdm(cdm)
}
\arguments{
\item{cdm}{A CDM reference object created by \code{patientsCDM()} with a
remote database backend.}
}
\value{
Invisibly returns NULL.
}
\description{
Drops the schema containing the test CDM and disconnects from the database.
}
